/**************************************************************************************************/
// ★desktop.js
//
// ▼メモ
//  - TODO・検討箇所がある場合、記述。
//
// ▼更新履歴
// 2015.02.23 表示値を全レコード→クエリ結果の全レコードの数値に変更
//            表示値をカンマ区切りに対応
// 2014.12.26 新規作成(Makoto.T)
//                                                           Copyright(C)2014 TIME CONCIER Co.,Ltd.
/**************************************************************************************************/
(function (PLUGIN_ID) {

    "use strict";

/**************************************************************************************************/
// イベントハンドラ
/**************************************************************************************************/

    /******************************************************/
    // 一覧画面"表示"のタイミングで実行
    /******************************************************/
    function initViewPage (event) {

        var records = event.records;

        // データがない・カレンダービューの場合は処理しない
        if ( !records.length ) { return; }

        // 一番上の行のクローンを生成
        var elTbody = document.getElementsByTagName("tbody")[0];
        var elCloneRow = elTbody.childNodes[0].cloneNode(true);
        elCloneRow.className = "tc_TotalRow";

        // 生成したクローン行の初期化(表示値をクリア)
        for ( var cnt1 in elCloneRow.childNodes ) {
            if ( isNaN(cnt1) ) { continue; }
            var elCell = elCloneRow.childNodes[cnt1].childNodes[0];

            if ( cnt1 == 0 ) { elCell.parentNode.removeChild(elCell); }	// 初回列の場合、コメントが付いていると子ノード(の子ノード)が複数になる為子ノード毎消す
            if ( elCell.firstChild ) {
                elCell.removeChild( elCell.firstChild );				// 表示内容の要素を削除
            }


        }
        elCell.removeChild( elCell.firstChild );						// 右端のごみ箱ボタンを消す

        // 全レコード情報から求めた合計値をクローン行の数値列に表示する
        var manager = new KintoneRecordManager;
        manager.getRecords( function(records) {


            for ( var key in records[0] ) {

                if ( records[0][key]['type'] == "NUMBER" ) {

                    // 数値型フィールドコードを取得
                    var fldCode = key;
                    // フィールドコードから取得したクラス名と一致する要素
                    var elFld = kintone.app.getFieldElements(fldCode);
                    if ( !elFld ) { continue; }
                    var elCell = elCloneRow.getElementsByClassName( elFld[0].className )[0];
                    // 表示内容を構成して1行目の対応するセルに追加する
                    var elSpan = document.createElement("span");
                    var sumVal = 0;
                    for ( var cnt2 in records ) {
                        if ( !isNaN(cnt2) ) { sumVal += Number(records[cnt2][fldCode]['value']); }
                    }		// 合計値取得

                    elSpan.textContent = String(sumVal).replace( /(\d)(?=(\d\d\d)+(?!\d))/g, '$1,' );		// 正規表現でカンマ区切り
                    elCell.childNodes[0].appendChild(elSpan);
                }
            }

        });

        // 先頭行に追加(※すでにある場合は処理しない)
        if ( !document.getElementsByClassName("tc_TotalRow")[0] ) {
            elTbody.insertBefore(elCloneRow, elTbody.firstChild);
        }

        return event;

    }

    //********************************/
    // イベントハンドラ設定
    //********************************/
    kintone.events.on(['app.record.index.show'], initViewPage);

/**************************************************************************************************/
// コール用関数・クラス
/**************************************************************************************************/

    /*
    ** kintoneと通信を行うクラス
    */
    var KintoneRecordManager = (function() {
        KintoneRecordManager.prototype.records = [];	// 取得したレコード
        KintoneRecordManager.prototype.appId = null;	// アプリID
        KintoneRecordManager.prototype.query = '';		// 検索クエリ
        KintoneRecordManager.prototype.limit = 100;		// 一回あたりの最大取得件数
        KintoneRecordManager.prototype.offset = 0;		// オフセット

        function KintoneRecordManager() {
            this.appId = kintone.app.getId();
        }

        // 開いている一覧のクエリ結果の全レコードを取得する
        KintoneRecordManager.prototype.getRecords = function(callback) {
            kintone.api(
                kintone.api.url("/k/v1/records", true),
                "GET",
                {
                    "app": this.appId,
                    "query": kintone.app.getQueryCondition() + (" limit " + this.limit + " offset " + this.offset)
                },
                (function(_this) {
                    return function(res) {
                        var len;
                        Array.prototype.push.apply(_this.records, res.records);
                        len = res.records.length;
                        _this.offset += len;
                        if (len < _this.limit) { // まだレコードがあるか？
                            _this.ready = true;
                            if (callback !== null) {
                                callback(_this.records); // レコード取得後のcallback
                            }
                        } else {
                            _this.getRecords(callback); // 自分自身をコール
                        }
                    };
                })(this)
            );
        };
        return KintoneRecordManager;
    })();


})(kintone.$PLUGIN_ID);
